//+------------------------------------------------------------------+
//|                                                       makeEA.mq4 |
//|                                             Copyright  2012, E1 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, E1"
#property link      "http://www.hlaiman.com"
#property show_inputs

#define S_OK 0
#define S_INIT "Strategy generator initialization "
#define S_MAKE "Strategy generation "
#define TIM_ERR 0.1 // bars time error 10%
#define BUY_ST "buy"
#define SELL_ST "sell"

#import  "mt4.dll"
  int InitEx(string symbol, int period, string name, int count);
  int PatternEx(string symbol, int period, string name, double& data[], int out, int in, int index); 
  int DeinitEx(int count);
  int SaveEx(string symbol, int period, int layers, int neurons, int epoch, int count);
#import
//+------------------------------------------------------------------+
extern string  ExpertFileName     = "";    //Expert advisor
extern int     PatternBarsCount   = 12;    //Bars on pattern
extern int     NetLayersCount     = 3;     //Neural layers
extern int     NetNeuronsCount    = 60;    //Neurons count
extern int     NetEpochCount      = 10000; //Learning epoch 
extern bool    Normalize          = false; //Normalization 
//+------------------------------------------------------------------+
static int LastArrow, PatternsCount;
static double makePattern[];
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
{
//----
    string exname = ExpertFileName;
    int obj_total = ObjectsTotal();
    if(StringLen(exname) <= 0) exname = TerminalName();
    if (obj_total <= 0) { Comment("Error, orders arrow not found!"); return(-1); }
    ArrayResize(makePattern, PatternBarsCount + 1);
    if(InitEx(Symbol(), Period(), exname, 1) == S_OK)
    Print(S_INIT, "OK - orders arrow found ", obj_total); else {
      Print(S_INIT, "Error!");
      return(-1);
    }
    PatternsCount = 0;
    LastArrow = 0;
    for (int i = 0; i < obj_total; i++)
    if(ObjectType(ObjectName(i)) == OBJ_ARROW) {
      Print("Arrow Detected");
      if(FillPattern(ObjectGet(ObjectName(i), OBJPROP_TIME1)) != true) Print("Aborted - Invalid pattern!");
      else {
        string onum = "", oname = ObjectName(i);
        if(ObjectGet(oname, OBJPROP_ARROWCODE) == 1) {
          if(StringGetChar(oname, 0) == '#') {
            for(int ii = 1; ii < StringLen(oname); ii++) {
              int ch = StringGetChar(oname, ii);
              if(ch >= '0' && ch <= '9') onum = onum + CharToStr(ch); else break;
            }
            if(StringLen(onum) > 0) {
              int anum = StrToInteger(onum);
              if(LastArrow != anum) {
                if(StringFind(oname, SELL_ST) > 0) {
                  makePattern[0] = OP_SELL;
                  PatternEx(Symbol(), Period(), ObjectName(i), makePattern, 1, ArraySize(makePattern) - 1, 0);
                  PatternsCount++;
                  Print("Sell signal detected at pattern #",PatternsCount);
                } else if(StringFind(oname, BUY_ST) > 0) {
                  makePattern[0] = OP_BUY;
                  PatternEx(Symbol(), Period(), ObjectName(i), makePattern, 1, ArraySize(makePattern) - 1, 0);
                  PatternsCount++;
                  Print("Buy signal detected at pattern #",PatternsCount);
                }
              }
            }
          }
        } else if(ObjectGet(oname, OBJPROP_ARROWCODE) == SYMBOL_ARROWDOWN) {
        // arrow down - sell
          makePattern[0] = OP_SELL;
          PatternEx(Symbol(), Period(), ObjectName(i), makePattern, 1, ArraySize(makePattern) - 1, 0);
          PatternsCount++;
          Print("Sell signal detected at pattern #",PatternsCount);
        } else if(ObjectGet(oname, OBJPROP_ARROWCODE) == SYMBOL_ARROWUP) {
        // arrow up - buy
          makePattern[0] = OP_BUY;
          PatternEx(Symbol(), Period(), ObjectName(i), makePattern, 1, ArraySize(makePattern) - 1, 0);
          PatternsCount++;
          Print("Buy signal detected at pattern #",PatternsCount);
        }
      }
    }
    if(PatternsCount > 0) {
      if(SaveEx(Symbol(), Period(), NetLayersCount, NetNeuronsCount, NetEpochCount, 1) >= 1)
      Print(S_MAKE, "Success! On ",PatternsCount," patterns"); else
      Print(S_MAKE, "Error! On ",PatternsCount," patterns");
    }	
    DeinitEx(1);
//----
  return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool FillPattern(datetime tim = 0)
{
  double hi, lo, op, cl;
  int i, ii = 1, ind = 1, iend;
  if(tim > 0) while(tim < Time[ind]) ind++;
  iend = ind + ArraySize(makePattern) - 1; 
//  for(i = ind; i <= iend; i++) // Validate bars time
//  if(MathAbs(Time[i - 1] - Time[i] - Period() * 60) > (Period() * 60 * TIM_ERR)) return(false);
  while(ind <= iend) {
    hi = High[ind];
    lo = Low[ind];
    op = Open[ind];
    cl = Close[ind];
    if((hi - lo) <= 0)  return(false); else
     makePattern[ii] = (((lo - op) - (hi - cl)) / (hi - lo));
//    makePattern[ii] = (((cl - lo) - (hi - cl)) / (hi - lo)) / 2 + 0.5;
    ind++;
    ii++;
  }
  if(Normalize == true) Normalize();
  return(true);
}
//+------------------------------------------------------------------+

void Normalize()
{
  int i;
  double ma, mi, he;
  ma = makePattern[1];
  mi = makePattern[1];
  for(i = 2; i < ArraySize(makePattern); i++) {
    if (ma < makePattern[i]) ma = makePattern[i];
    if (mi > makePattern[i]) mi = makePattern[i];
  }
  he = 2 / (ma - mi);
  for(i = 1; i < ArraySize(makePattern); i++)
  makePattern[i] = ((makePattern[i] - mi - 1) * he); 
}

//+------------------------------------------------------------------+